#!/usr/bin/env perl
use utf8;
use strict; use warnings; use v5.32;
use Text::ParseWords qw/parse_line quotewords/;
STDOUT->autoflush(1); STDERR->autoflush(1);

my $cardnum = 0;

sub run_cmd($$$$) { 
  # Runs amixer command; returns chomped stdout/stderr or undef on failure
  my ($numid, $name, $op, $param) = @_;
  my $cmd = "amixer -c $cardnum $op numid=$numid $param 2>&1";
  #say "> $cmd"; ###TEMP
  my $output = qx/$cmd/; while (chomp $output) {}
  if ($? != 0) {
    die "$cmd [name=$name]\nFailed to execute: $!\n" if $? == -1;
    die "$cmd [name=$name]\nDied with signal ",($? & 127),"\n" if ($? & 127);
    say "$cmd [name=$name]\nFAILED (exit ",($? >> 8),"): «$output»";
    return undef
  } else {
    return $output;
  }
}

###########
# Get list of devices
###########
my @devices;
foreach my $line (qx/amixer -c $cardnum controls/) {
  chomp $line;
  my %h;
  foreach( parse_line(',', 0, $line) ) {
    /^(\w+)=(.+)$/ or die "INVALID PART <$_> in line:$line\n";
    $h{$1} = $2;
  }
  die "NO 'numid' in:$line\n" unless $h{numid};
  die "NO 'name' in:$line\n" unless $h{name};
  if ($h{name} =~ /Jack|Pro Mask|Con Mask|Channel Map|Default|Boost|Mode/
      or $h{name} eq "ELD"
     ) {
    say "--SKIPPING numid=$h{numid} name=$h{name}";
    next
  }
  push @devices, \%h;
}

###########
# Get current settings
###########
my %oldvals;  # numid => oldval
foreach my $h (@devices) {
  my $numid = $h->{numid};
  my $name  = $h->{name};
  my $s = run_cmd($numid, $name, "cget", "") // die "cget failed";
  # Note .* does not match \n (without /s)
  $s =~ /: values=(.*)/ or die "UNEXPECTED RESULT from cget $numid $name:$s";
  say "Initially $name ($numid) is '$1'";
  $oldvals{$numid} = $1;
}

###########
# Turn them off
###########
foreach my $h (@devices) {
  my $numid = $h->{numid};
  my $name  = $h->{name};
  my $oldval = $oldvals{$numid};
  my $newval = ( # Note, the "value" may be a comma-separated list
       $oldval =~ /true|false/i ? ($oldval =~ s/true/false/igr) :
       $oldval =~ /on|off/i     ? ($oldval =~ s/on/off/igr) :
       $oldval =~ /\d/i         ? ($oldval =~ s/\d+/0/igr) :
       die("CANT GROK oldval '$oldval' for $name (numid $numid)")
  );
  my $s = run_cmd($numid, $name, "cset", $newval) // die "cset failed";
  say "Successfully turned off $name (set to $newval) [$numid]";
} 

###########
# Wait for user to finish
###########
print "\nShould be turned off now.\n";
print "Press ENTER to restore settings: "; STDOUT->flush;
<STDIN>;

END{ # END blocks run on exit, including if interrupted by a signal
  ###########
  # Restore previous settings
  ###########
  foreach my $h (@devices) {
    my $numid = $h->{numid};
    my $name  = $h->{name};
    my $newval = $oldvals{$numid};
    next unless defined $newval; # aborted part way thru?
    my $s = run_cmd($numid, $name, "cset", $newval) // die "restore failed";
    say "Successfully restored $name to $newval [numid=$numid]";
  }
}

exit 0;
